package cn.com.ty.lift.business.capital.service;

import cn.com.ty.lift.business.capital.dao.entity.CapitalRepair;
import cn.com.ty.lift.business.capital.dao.entity.model.request.CapitalRepairAuditRequest;
import cn.com.ty.lift.business.capital.dao.entity.model.request.CapitalRepairExtensionRequest;
import cn.com.ty.lift.business.capital.dao.entity.model.request.CapitalRepairRequest;
import cn.com.ty.lift.business.capital.dao.entity.model.response.CapitalRepairResponse;
import cn.com.ty.lift.business.capital.dao.mapper.CapitalRepairMapper;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.project.dao.entity.ProjectUser;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectUserResponse;
import cn.com.ty.lift.business.project.service.ProjectUserService;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.com.ty.lift.common.constants.CommonEnum.CapitalRepairStatus;

/**
 * @author bieao
 * @date 2020/2/3 11:38 上午
 * @description 大修项目业务层
 */
@Service
public class CapitalRepairService extends ServiceImpl<CapitalRepairMapper, CapitalRepair> {

    @Resource
    private ProjectUserService projectUserService;

    /**
     * @param request 大修项目列表查询条件
     * @return RestResponse 结果集
     * @description 条件查询大修项目列表
     * @date 2020/2/3 11:38 上午
     */
    public IPage<CapitalRepairResponse> list(CapitalRepairRequest request) {
        IPage<CapitalRepairResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return baseMapper.findByCondition(page, request);
    }

    /**
     * @param id 大修项目id
     * @return CapitalRepair 大修项目详情
     * @description 查询大修项目详情
     * @date 2020/2/25 11:58 上午
     */
    public CapitalRepair getOne(Long id) {
        QueryWrapper<CapitalRepair> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<CapitalRepair> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(CapitalRepair::getId, id);
        return getOne(lambdaQueryWrapper);
    }

    /**
     * @param id 大修项目id
     * @return Map<String, Object> 大修项目详情和项目组列表
     * @description 查询大修项目详情和项目组列表
     * @date 2020/2/3 11:31 上午
     */
    public Map<String, Object> detail(Long id) {
        CapitalRepair capitalRepair = getOne(id);
        List<ProjectUserResponse> userList = projectUserService.getUserList(id);
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("capitalRepair", capitalRepair);
        resultMap.put("userList", userList);
        return resultMap;
    }

    /**
     * @param request 新增大修项目数据项
     * @return Integer 成功1或失败0
     * @description 新增大修项目
     * @date 2020/2/3 11:31 上午
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse add(CapitalRepairExtensionRequest request) {
        CapitalRepair repair = request.getCapitalRepair();
        String projectCode = DateUtils.generateCode();
        repair.setProjectCode("XM" + projectCode);
        repair.setProjectStatus(repair.getStartDate().isAfter(LocalDate.now()) ? "1" : "2");
        boolean result = save(repair);
        if (!result) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        //批量新增项目成员信息
        List<ProjectUser> users = request.getUserList();
        users.forEach(user -> user.setProjectId(repair.getId()));
        boolean ret = projectUserService.saveBatch(users, users.size());
        if (!ret) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param capitalRepair 修改大修项目数据项
     * @return 是否成功
     * @description 修改大修项目
     * @date 2020/2/3 11:36 上午
     */
    public boolean modify(CapitalRepair capitalRepair) {
        return updateById(capitalRepair);
    }

    /**
     * @param request 是否通过
     * @return Integer 成功1或失败0
     * @description 大修审核
     * @date 2020/2/4 12:46 下午
     */
    public boolean audit(CapitalRepairAuditRequest request) {
        CapitalRepair capitalRepair = new CapitalRepair();
        capitalRepair.setId(request.getId());
        capitalRepair.setProjectStatus("0".equals(request.getPass()) ?
                CapitalRepairStatus.ABUILDING.getCode() : CapitalRepairStatus.COMPLETE.getCode());
        return updateById(capitalRepair);
    }

}
